##################################
### Split.tcl                  ###
### Version 4.1                ###
### By Wcc                     ###
### wcc@techmonkeys.org        ###
### http://www.dawgtcl.com:81/ ###
### EFnet #|DAWG|Tcl           ###
##################################

############################################################################
### Copyright  2000 - 2002 |DAWG| Scripting Group. All rights reserved. ###
############################################################################

###########################################################################
## This script checks which servers are linked to your irc network       ##
## every x seconds. It can also check the status of hidden U:lined       ##
## servers, such as Services and StatServ. When a server splits, the     ##
## bot will open a socket to the split server, and report whether the    ##
## connection was either: Accepted, Refused, Failed, or Timed out. This  ##
## information is also created when you use the status command from a    ##
## channel or from the partyline. All data is saved to database files.   ##
## You can set flags separately for different types of commands.         ##
##                                                                       ##
## The script also optionally creates an HTML web page containing: a     ##
## list of all linked servers with various information about each one,   ##
## a list of all split servers, information about the network's users,   ##
## a graph showing most frequently split servers, a network map, and a   ##
## log of recent network activity. Its style is fully configurable, and  ##
## it is updated whenever information changes or when you use the .split ##
## HTML command from the partyline. The network map can also be viewed   ##
## from the partyline. You can also optionally display servers from some ##
## or all sections of the page as mIRC links.                            ##
##                                                                       ##
## Thanks to KuNgFo0, guppy, Bass, MatriX-, paulsen, and stdarg for all  ##
## of their help.                                                        ##
###########################################################################

##############
## COMMANDS ##
#########################################################
## DCC ## .split <command> (Can be changed)            ##
######### Use .split help for a full list of commands. ##
######### -------------------------------------------- ##
######### .chanset <channel> +/-split                  ##
######### This enables or disables split notification  ##
######### for a channel.                               ##
#########################################################
## PUB ## !split <command> (Can be changed)            ##
######### Use !split help for a full list of commands. ##
#########################################################

##########################################################
## Just load the script, set the variables, and rehash. ##
## Read all comments carefully.                         ##
##########################################################

#####################
## General Options ##
#####################

#######################################################
# Set the main dcc command for using the script here. #
#######################################################

set split_setting(command_dcc) "split"

#######################################################
# Set the main pub command for using the script here. #
#######################################################

set split_setting(command_pub) "!split"

#############################################################################
# Set the pub command that allows all users to view the netsplit list here. #
# NOTE: To disable this feature, simply set this to "".                     #
#############################################################################

set split_setting(nflagtrig) "!splits"

#####################################################################
# Set the flag required for manually updating the server list here. #
#####################################################################

set split_setting(flag_links) "o|o"

#########################################################
# Set the flag for checking the status of servers here. #
#########################################################

set split_setting(flag_status) "o|o"

#########################################################################
# Set the flag required for running maintenance functions of the script.#
#########################################################################

set split_setting(flag_maintenance) "o|o"

###############################################################
# Set the flag required for listing netsplits and viewing the #
# network map and links lists here.                           #
###############################################################

set split_setting(flag_splits) "o|o"

##########################################################################
# Set the interval in seconds for automatically updating the server list #
# and saving databases here.                                             #
#                                                                        #
# NOTE: Do not set this too low. IRCOps on IRC networks such as EFnet or #
# DALnet can view /links requests, and possibly K:Line your bot.         #
##########################################################################

set split_setting(t_update) "300"

##############################################################################
# Check for splits from services.* and stats.*?                              #
#                                                                            #
# This works by sending /VERSION to the servers, as these servers are hidden #
# from /LINKS. Since /VERSION commands can not be seen by IRC Operators,     #
# you can set the timer for this command to a much lower time without worry  #
# of your bot getting K:Lined.                                               #
#                                                                            #
# Services are often very lagged immediately after rejoining to the network, #
# thus delaying the time it takes to send a version reply to the bot. A      #
# rejoin will not be detected until after services actually sends the bot    #
# the version reply.                                                         #
##############################################################################

set split_setting(services_check) 0

###########################################################################
# Set the names of the services server here. Leave as default for DALnet. #
# Set to "" to disable. Server names are case sensitive.                  #
###########################################################################

set split_setting(services_servicesname) "services.dal.net"

########################################################################
# Set the names of the stats server here. Leave as default for DALnet. #
# Set to "" to disable. Server names are case sensitive.               #
########################################################################

set split_setting(services_statsname) "stats.dal.net"

####################################################
# Broadcast a notice when a user enters a command? #
####################################################

set split_setting(cmdcast) 1

############################################
# Broadcast a notice when a server splits? #
# 1 = broadcast, 2 = log, 3 = silent.      #
############################################

set split_setting(notify_cast) 1

##############################################
# Log all commands to the eggdrop's logfile? #
##############################################

set split_setting(cmdlog) 0

############################################################
# Set the default port to connect to when a server splits. #
############################################################

set split_setting(default_port) "6667"

#######################
## Web Page Settings ##
#######################

######################################################################
# Set here the path to the DAWG template file you would like to use. #
# You can obtain templates by visiting http://dawg.oc255.net:81/ and #
# clicking the templates link.                                       #
######################################################################

set split_setting(tpl) "./scripts/Split_DAWG.tpl"

#######################################
# Completely disable web page output? #
#######################################

set split_setting(html_disable) 0

###############################
# Set the HTML filename here. #
###############################

set split_setting(netstats_page) "../web/efnet.html"

###############################
## Log and Database Settings ##
###############################

#################################################
# Set the path to store database files in here. #
#################################################

set split_setting(db_path) "./scripts/"

################################################
# Automatically clear the logfile at midnight? #
################################################

set split_setting(auto_clearlog) 1

###################################################
# Automatically backup the databases at midnight? #
###################################################

set split_setting(auto_backup) 1

##############
## Messages ##
##############

###################################
# Enable use of bold in dcc chat? #
###################################

set split_setting(bold) 1

#########################################
# Prefix "SPLIT:" in dcc chat messages? #
#########################################

set split_setting(SPLIT:) 1

#####################################################################
# Set the message for a Rejoin here. "%server" is the name of the   #
# server that was split, "%splitfrom" is the name of the server it  #
# split from, "%time" is the time the split happened, and %duration #
# is how long the split lasted.                                     #
#####################################################################

set split_setting(rejoin_prefix) "Rejoin: %server to %splitfrom (%time) (Split lasted %duration)."

########################################################################
# Set the message for a New Link (Server added to the network) here.   #
# "%server" is the name of the new server, and "%linkedto" is the name #
# of the server it is linked to.                                       #
########################################################################

set split_setting(newlink_prefix) "New Link: %server to %linkedto."

###############################################################################
# Set the message for a Relink (Server is linked to a different server) here. #
# "%server" is the name of the server that was relinked, "%oldlink" is the    #
# server it was linked to, and "%newlink" is the server it is now linked to.  #
###############################################################################

set split_setting(relink_prefix) "Relink: %server from %oldlink to %newlink."

#####################################################################
# Set the message for a netsplit with an accepted connection here.  #
# "%server" is the name of the split server. %splitfrom is the name #
# of the server it split from.                                      #
#####################################################################

set split_setting(connaccept_msg) "Netsplit: %server from %splitfrom. Connection to %server was accepted."

#####################################################################
# Set the message for a netsplit with a timed out connection here.  #
# "%server" is the name of the split server. %splitfrom is the name #
# of the server it split from.                                      #
#####################################################################

set split_setting(conntimeout_msg) "Netsplit: %server from %splitfrom. Connection to %server timed out."

#####################################################################
# Set the message for a netsplit with a failed connection here.     #
# "%server" is the name of the split server. %splitfrom is the name #
# of the server it split from.                                      #
#####################################################################

set split_setting(connfail_msg) "Netsplit: %server from %splitfrom. Connection to %server failed."

#####################################################################
# Set the message for a netsplit with a refused connection here.    #
# "%server" is the name of the split server. %splitfrom is the name #
# of the server it split from.                                      #
#####################################################################

set split_setting(connrefused_msg) "Netsplit: %server from %splitfrom. Connection to %server was refused."

####################
# Code begins here #
####################

if {![string match 1.6.* $version]} { putlog "\002SPLIT:\002 \002WARNING:\002 This script is intended to run on eggdrop 1.6.x or later." }
if {[info tclversion] < 8.2} { putlog "\002SPLIT:\002 \002WARNING:\002 This script is intended to run on Tcl Version 8.2 or later." }

foreach split_rbind [list 251 252 254 255 265 266 351 364 365 402] { bind raw - $split_rbind split_raw }
foreach split_rbind [list init-server connect-server] { bind evnt - $split_rbind split_evnt}
bind dcc - $split_setting(command_dcc) split_split_dcc
bind pub - $split_setting(command_pub) split_split_pub
bind time - "59 23 * * *" split_run_maintenance
bind pub - $split_setting(nflagtrig) split_nflaglist
if {$split_setting(nflagtrig) == ""} { unbind pub - $split_setting(nflagtrig) split_nflaglist }

setudef flag split

set split_setting(version) "4.1"
set split_notpl 0
set split_setting(debug) 0
if {![info exists split_init_server]} { set split_init_server 0 }
if {![info exists split_connect_server]} { set split_connect_server 0 }

proc split_debug {line {text ""}} {
	global split_setting
	if {!$split_setting(debug)} { return }
	putlog "[split_dopre]DEBUG: Log: $line - Passed."
	if {$text != ""} { putlog "[split_dopre]DEBUG: Log: $line: $text" }
}
proc split_readarray {array file} {
	upvar $array ours
	if {[catch {open $file r} fd]} { return }
	if {[array exists ours]} { unset ours }
	foreach line [split [read $fd] \n] {
		if {[string range $line 0 2] != "%!%" || [scan [string range $line 3 end] "%\[^!!!\]!!!%s" name data] <= 0} { continue }
		if {![info exists data]} { set data "" }
		set ours($name) $data
	}
	close $fd
}
proc split_savearray {array file} {
	upvar $array ours
	if {[catch {open $file w} fd] || ![array exists ours]} { return }
	foreach entry [array names ours] { puts $fd "%!%$entry!!!$ours($entry)" }
	close $fd
}
proc split_dopre {} {
	global split_setting
	if {!$split_setting(SPLIT:)} { return "" }
	if {!$split_setting(bold)} { return "SPLIT: " }
	return "\002SPLIT:\002 "
}
proc split_format_td {type line center text} {
	global split_setting
	switch -- $type {
		"title" { return "<font size=\"$split_setting(Page_THS)\" color=\"$split_setting(Page_TextC)\">[expr ($center)?{<center>}:{}]$text[expr ($center)?{</center>}:{}]</font>" }
		"header" { return "<td align=\"left\" bgcolor=\"$split_setting(Page_THC_${line})\"><font size=\"$split_setting(Page_THS)\" color=\"$split_setting(Page_HC)\">[expr ($center)?{<center>}:{}]$text[expr ($center)?{</center>}:{}]</font></td>" }
		"cell" { return "<td align=\"left\" bgcolor=\"$split_setting(Page_TCC_${line})\"><font size=\"$split_setting(Page_TCS)\" color=\"$split_setting(Page_CC)\">[expr ($center)?{<center>}:{}]$text[expr ($center)?{</center>}:{}]</font></td>" }
	}
}
proc split_logcast {text} {
	global split_setting
	if {$split_setting(cmdcast)} { dccbroadcast "[split_dopre]$text" }
	if {$split_setting(cmdlog)} { split_putlog "$text" }
}
proc split_notify {text} {
	global split_setting
	if {$split_setting(notify_cast) == 1} {
		dccbroadcast "[split_dopre]$text"
	} elseif {$split_setting(notify_cast) == 2} {
		putlog "[split_dopre]$text"
	}
	foreach chan [channels] {
		if {[botonchan $chan] && [lsearch -exact [channel info $chan] +split] != -1} { split_putchan $chan $text }
	}
}
proc split_queue {i} { return [expr ($i)?"putserv":"puthelp"] }
proc split_lusersleft {rsub text} {
	if {$rsub} { regsub -all -- ":" $text "" text }
	return [string trimleft [join [lrange [split $text] 1 end]] :]
}
proc split_notnumber {string} { return [regexp {[^0-9]} $string] }
proc split_mIRC_link {section server} {
	global split_setting
	if {$split_setting(mIRC_$section) && ![split_cm $server Unknown]} { return "<a href=\"irc://$server\">$server</a>" } { return $server }
}
proc split_checks {type} {
	global split_setting
	if {([split_cm $type services] && $split_setting(services_servicesname) != "") || ([split_cm $type stats] && $split_setting(services_statsname) != "") || ([split_cm $type all] && $split_setting(services_statsname) != "" && $split_setting(services_servicesname) != "")} { return 1 } { return 0 }
}
proc split_putlog {text} { putlog "[split_dopre]$text" }
proc split_putdcc {idx text} { putdcc $idx "[split_dopre]$text" }
proc split_putnotc {nick text} { puthelp "NOTICE $nick :$text" }
proc split_putchan {chan text} { putserv "PRIVMSG $chan :$text" }
proc split_cm {pattern string} { return [string match $pattern $string] }
proc split_ncm {pattern string} { return [split_cm [string tolower $pattern] [string tolower $string]] }
foreach d {masterservers linkedservers splittime connectionstatus splitstats log services lusers_data} { set split_database($d) ${split_setting(db_path)}${d}.db }
foreach a {linkedservers connectionstatus splitstats splittime masterservers lusers_data} {
	if {![array exists split_$a] && [file exists $split_database($a)]} { split_readarray split_$a $split_database($a) }
}
if {[lsearch -glob [utimers] "* split_update *"] == -1} { utimer $split_setting(t_update) split_update }
if {$split_setting(services_check) && ![array exists split_services_split] && [file exists $split_database(services)]} { split_readarray split_services_split $split_database(services) }
if {![info exists split_splittime(start)]} {
	set split_splittime(start) [clock seconds]
	split_savearray split_splittime $split_database(splittime)
}
if {![info exists split_setting(tpl)] || [catch {source $split_setting(tpl)}]} {
	putlog "\002SPLIT:\002 The template file could not be opened; using default."
	set split_notpl 1
}
foreach {s v} [list Page_CC #000000 Page_HC #000000 Page_LinkC #0000FF Page_TTextC #000000 Page_TextC #000000 Page_BGC \
	#FFFFFF Page_TCC_2 #E0E0E0 Page_TCC_1 #C0C0C0 Page_THC_2 #8080E0 Page_THC_1 #8080C0 Page_TCS 4 Page_THS 5 Page_TitleS 6 Page_Title \
	{Network Status} Page_TitleTag {Network Status} Page_Img {} mIRC_Network_Map 1 mIRC_Linked_Servers 1 mIRC_Split_Servers 1 mIRC_Server_Stats \
	1 Page_Activity_Log 1 Page_Network_Map 1 Page_Network_Users 1 Page_Linked_Servers 1 Page_Split_Servers 1 Page_Split_Graph 1 Page_GImg {}] {
	if {$split_notpl || ![info exists split_setting($s)]} { set split_setting($s) $v  }
}
proc split_evnt {type} {
	global split_connect_server split_init_server split_connect_server
	if {[split_ncm $type connect-server]} { set split_connect_server 1 }
	if {![split_ncm $type init-server]} { return }
	set split_connect_server 0
	set split_init_server 1
	putserv "LINKS"
}
proc split_split_dcc {hand idx text} {
	global split_init_server split_connect_server split_localserver split_services_split split_database split_splitstats split_setting split_masterservers split_linkedservers split_splittime split_connectionstatus
	if {![matchattr $hand $split_setting(flag_maintenance)] && ![matchattr $hand $split_setting(flag_splits)] && ![matchattr $hand $split_setting(flag_links)]} { putdcc $idx "What? You need '.help'" ; return }
	if {![info exists split_masterservers] || ![info exists split_linkedservers] || $split_init_server || $split_connect_server} { split_putdcc $idx "The server databases do not exist. Please wait until the next automatic update occurs." ; return }
	for {set i 0} {$i <= 2} {incr i} { set arg${i} [lindex [split $text] $i] }
	if {$arg0 == "" || ([split_ncm help $arg0] && $arg1 == "")} {
		split_putdcc $idx "Usage: .$split_setting(command_dcc) <command>"
		split_putdcc $idx "Commands:"
		if {[matchattr $hand $split_setting(flag_maintenance)]} {
			split_putdcc $idx "Backup - Backs up all databases and logfiles."
			split_putdcc $idx "ClearLogs - Clears the activity log and refreshes the web page."
			split_putdcc $idx "DelSplit - Deletes a netsplit from the list of splits."
		}
		split_putdcc $idx "Help - Shows the command list for Split.tcl."
		if {[matchattr $hand $split_setting(flag_maintenance)]} { split_putdcc $idx "HTML - Refreshes the generated web page." }
		if {[matchattr $hand $split_setting(flag_links)]} { split_putdcc $idx "Links - Manually updates the server list." }
		if {[matchattr $hand $split_setting(flag_splits)]} {
			split_putdcc $idx "List - Displays a list of all current netsplits."
			split_putdcc $idx "Map - Displays a map of the network."
		}
		if {[matchattr $hand $split_setting(flag_status)]} { split_putdcc $idx "Status - Displays information on a server." }
		split_putdcc $idx "See .$split_setting(command_dcc) help <command> for more information."
	} elseif {[split_ncm help $arg0]} {
		if {[split_ncm html $arg1] && [matchattr $hand $split_setting(flag_maintenance)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) HTML"
			split_putdcc $idx "This command manually updates the web page generated by the script."
		} elseif {[split_ncm clearlog* $arg1] && [matchattr $hand $split_setting(flag_maintenance)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) ClearLogs"
			split_putdcc $idx "This command clears the network activity log and updates the netstats web page."
		} elseif {[split_ncm backup $arg1] && [matchattr $hand $split_setting(flag_maintenance)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) Backup"
			split_putdcc $idx "This command backs up all logs and databases."
		} elseif {[split_ncm list $arg1] && [matchattr $hand $split_setting(flag_splits)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) List"
			split_putdcc $idx "This displays a list of all current netsplits."
		} elseif {[split_ncm del* $arg1] && [matchattr $hand $split_setting(flag_maintenance)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) DelSplit <server>"
			split_putdcc $idx "This command deletes a netsplit from the list of splits. It can be useful when a server is renamed or delinked."
		} elseif {[split_ncm links $arg1] && [matchattr $hand $split_setting(flag_links)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) Links"
			split_putdcc $idx "This updates the server list by sending /LINKS to the server. If services checking is enabled, this will also send /VERSION to services and stats."
		} elseif {[split_ncm map $arg1] && [matchattr $hand $split_setting(flag_splits)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) Map"
			split_putdcc $idx "This displays a map of the network, similar to /LINKS in mIRC."
		} elseif {[split_ncm status $arg1] && [matchattr $hand $split_setting(flag_status)]} {
			split_putdcc $idx "Usage: .$split_setting(command_dcc) Status <server> \[port\]"
			split_putdcc $idx "You can use this command to get information on a server."
			split_putdcc $idx "Information includes:"
			split_putdcc $idx "The server's connection status"
			split_putdcc $idx "Whether or not the server is linked to the network"
			split_putdcc $idx "The server's uplink"
			split_putdcc $idx "The time, date, uplink, and connection status at the time of the split (if the server is split)"
			split_putdcc $idx "The number of times the server has split"
		} {
			split_putdcc $idx "No help is available for that command."
		}
	} elseif {[split_ncm clearlog* $arg0] && [matchattr $hand $split_setting(flag_maintenance)]} {
		split_logcast "$hand is clearing the log database."
		split_putdcc $idx "Clearing logs."
		if {[catch [file delete -force $split_database(log)]]} { split_putdcc $idx "The log file could not be deleted." }
		split_putsvrlog 5 "Clearing&log&database.&\($hand\)"
	} elseif {[split_ncm backup $arg0] && [matchattr $hand $split_setting(flag_maintenance)]} {
		split_logcast "$hand is backing up all logs and databases."
		split_putdcc $idx "Backing up all logs and databases."
		foreach d [array names split_database] {
			if {[catch [file copy -force $split_database($d) $split_database($d).bak]]} { split_putdcc $idx "Could not backup $split_database($d)." }
		}
	} elseif {[split_ncm del* $arg0] && [matchattr $hand $split_setting(flag_maintenance)]} {
		if {$arg1 == ""} { split_putdcc $idx "Usage: .$split_setting(command_dcc) DelSplit <server>" ; return }
		if {[info exists split_linkedservers($arg1)] || ![info exists split_masterservers($arg1)]} { split_putdcc $idx "$arg1 is not currently split." ; return }
		foreach i [list connectionstatus splittime masterservers uplink hops infoline] { catch { unset split_${i}($arg1) } }
		split_putsvrlog 5 "Removed&$arg1&from&the&split&server&list.&\($hand\)"
		split_logcast "$hand has deleted $arg1 from the netsplit list."
		split_putdcc $idx "$arg1 has been removed from the netsplit list."
	} elseif {[split_ncm html $arg0] && [matchattr $hand $split_setting(flag_maintenance)]} {
		split_logcast "$hand is updating the netstats web page."
		split_create_html
		split_putdcc $idx "The web page has been updated."
	} elseif {[split_ncm list $arg0] && [matchattr $hand $split_setting(flag_splits)]} {
		split_logcast "$hand requested a netsplit list."
		if {[array size split_masterservers] == [array size split_linkedservers] && (!$split_setting(services_check) || (![info exists split_services_split(stats)] && ![info exists split_services_split(stats)]) || ![split_checks all] || [expr ($split_services_split(services) + $split_services_split(stats)) == 0])} { split_putdcc $idx "There are no split servers." ; return }
		split_putdcc $idx "Current netsplits:"
		foreach s [array names split_masterservers] {
			if {![info exists split_linkedservers($s)]} { split_putdcc $idx "$s  \(from $split_masterservers($s)\) \(Connection: $split_connectionstatus($s)\) \(Split: [clock format $split_splittime($s) -format "%I:%M:%S %p \(%D\)"]\)" }
		}
		if {$split_setting(services_check)} {
			if {[info exists split_services_split(services)] && [split_checks services] && $split_services_split(services)} { split_putdcc $idx "$split_setting(services_servicesname) \(from \(Unknown\)\) \(Split: [clock format $split_splittime($split_setting(services_servicesname)) -format "%I:%M:%S %p \(%D\)"]\)" }
			if {[info exists split_services_split(stats)] && [split_checks stats] && $split_services_split(stats)} { split_putdcc $idx "$split_setting(services_statsname) \(from \(Unknown\)\) \(Split: [clock format $split_splittime($split_setting(services_statsname)) -format "%I:%M:%S %p \(%D\)"]\)" }
		}
		split_putdcc $idx "End of list."
	} elseif {[split_ncm map $arg0] && [matchattr $hand $split_setting(flag_splits)]} {
		split_logcast "$hand requested a network map."
		split_putdcc $idx "  $split_localserver"
		split_netmap split_putdcc "     " $idx $split_localserver
	} elseif {[split_ncm status $arg0] && [matchattr $hand $split_setting(flag_status)]} {
		if {$arg1 == ""} { split_putdcc $idx "Usage: .$split_setting(command_dcc) status <server> \[port\]" ; return }
		if {$arg2 == ""} { set arg2 $split_setting(default_port) }
		split_logcast "$hand is checking the status of $arg1 \($arg2\)."
		split_putdcc $idx "Checking the status of $arg1 \($arg2\)."
		if {[catch {set sock [socket -async $arg1 $arg2]}]} { split_status_output split_putdcc $idx $arg1 $arg2 "was refused." ; return	}
		fileevent $sock writable [list split_connected 1 0 $idx $sock $arg1 $arg2 [utimer 15 [list split_timeout 1 0 $idx $sock $arg1 $arg2]]]
	} elseif {[split_ncm links $arg0] && [matchattr $hand $split_setting(flag_links)]} {
		split_logcast "$hand requested a links update."
		putserv "LINKS"
		split_putdcc $idx "The links list has been updated."
	} {
		split_putdcc $idx "Usage: .$split_setting(command_dcc) <command>"
		split_putdcc $idx "See .$split_setting(command_dcc) help for a command list."
	}
}
proc split_split_pub {nick uhost hand chan text} {
	global split_init_server split_services_split split_database split_splitstats split_setting split_masterservers split_connect_server split_linkedservers split_splittime split_connectionstatus
	if {![matchattr $hand $split_setting(flag_maintenance)] && ![matchattr $hand $split_setting(flag_splits)] && ![matchattr $hand $split_setting(flag_links)]} { return }
	if {![info exists split_masterservers] || ![info exists split_linkedservers] || $split_init_server || $split_connect_server} { split_putchan $chan "The server databases do not exist. Please wait until the next automatic update occurs." ; return }
	for {set i 0} {$i <= 2} {incr i} { set arg${i} [lindex [split $text] $i] }
	if {$arg0 == "" || ([split_ncm help $arg0] && $arg1 == "")} {
		split_putnotc $nick "Usage: $split_setting(command_pub) <command>"
		split_putnotc $nick "Commands:"
		split_putnotc $nick "Help - Shows the command list for Split.tcl."
		if {[matchattr $hand $split_setting(flag_links)]} { split_putnotc $nick "Links - Manually updates the server list." }
		if {[matchattr $hand $split_setting(flag_splits)]} { split_putnotc $nick "List - Displays a list of all current netsplits." }
		if {[matchattr $hand $split_setting(flag_status)]} { split_putnotc $nick "Status - Displays information on a server." }
		split_putnotc $nick "See $split_setting(command_pub) help <command> for more information."
	} elseif {[split_ncm help $arg0]} {
		if {[split_ncm list $arg1] && [matchattr $hand $split_setting(flag_splits)]} {
			split_putnotc $nick "Usage: $split_setting(command_pub) List"
			split_putnotc $nick "This displays a list of all current netsplits."
		} elseif {[split_ncm links $arg1] && [matchattr $hand $split_setting(flag_links)]} {
			split_putnotc $nick "Usage: $split_setting(command_pub) Links"
			split_putnotc $nick "This updates the server list by sending /LINKS to the server. If services checking is enabled, this will also send /VERSION to services and stats."
		} elseif {[split_ncm status $arg1] && [matchattr $hand $split_setting(flag_status)]} {
			split_putnotc $nick "Usage: $split_setting(command_pub) Status <server> \[port\]"
			split_putnotc $nick "You can use this command to get information on a server."
			split_putnotc $nick "Information includes:"
			split_putnotc $nick "The server's connection status"
			split_putnotc $nick "Whether or not the server is linked to the network"
			split_putnotc $nick "The server's uplink"
			split_putnotc $nick "The time, date, uplink, and connection status at the time of the split (if the server is split)"
			split_putnotc $nick "The number of times the server has split"
		} {
			split_putnotc $nick "No help is available for that command."
		}
	} elseif {(([split_ncm html $arg0] || [split_ncm del* $arg0] || [split_ncm clearlog* $arg0]) && [matchattr $hand $split_setting(flag_maintenance)]) || ([split_ncm map $arg0] && [matchattr $hand $split_setting(flag_splits)])} {
		split_putnotc $nick "This command can only be used from the partyline."
	} elseif {[split_ncm list $arg0] && [matchattr $hand $split_setting(flag_splits)]} {
		split_logcast "$nick requested a netsplit list. \($chan\)"
		if {[array size split_masterservers] == [array size split_linkedservers] && (!$split_setting(services_check) || (![info exists split_services_split(stats)] && ![info exists split_services_split(stats)]) || ![split_checks all] || [expr ($split_services_split(services) + $split_services_split(stats)) == 0])} { split_putchan $chan "There are no split servers." ; return }
		set expr 0
		if {$split_setting(services_check)} {
			if {[info exists split_services_split(services)] && [split_checks services]} { set expr $split_services_split(services) }
			if {[info exists split_services_split(stats)] && [split_checks stats]} { set expr [expr $expr + $split_services_split(stats)] }
		}
		[split_queue [set queue [expr (([array size split_masterservers] - [array size split_linkedservers] + $expr) > 3)?0:1]]] "PRIVMSG $chan :Current netsplits:"
		foreach s [array names split_masterservers] {
			if {![info exists split_linkedservers($s)]} { [split_queue $queue] "PRIVMSG $chan :$s \(from $split_masterservers($s)\) \(Connection: $split_connectionstatus($s)\) \(Split: [clock format $split_splittime($s) -format "%I:%M:%S %p \(%D\)"]\)" }
		}
		if {$split_setting(services_check)} {
			if {[info exists split_services_split(services)] && [split_checks services] && $split_services_split(services)} { [split_queue $queue] "PRIVMSG $chan :$split_setting(services_servicesname) \(from \(Unknown\)\) \(Split: [clock format $split_splittime($split_setting(services_servicesname)) -format "%I:%M:%S %p \(%D\)"]\)" }
			if {[info exists split_services_split(stats)] && [split_checks stats] && $split_services_split(stats)} { [split_queue $queue] "PRIVMSG $chan :$split_setting(services_statsname) \(from \(Unknown\)\) \(Split: [clock format $split_splittime($split_setting(services_statsname)) -format "%I:%M:%S %p \(%D\)"]\)" }
		}
		[split_queue $queue] "PRIVMSG $chan :End of list."
	} elseif {[split_ncm status $arg0] && [matchattr $hand $split_setting(flag_status)]} {
		if {$arg1 == ""} { split_putnotc $nick "Usage: $split_setting(command_pub) status <server> \[port\]"; return }
		if {$arg2 == ""} { set arg2 $split_setting(default_port) }
		split_logcast "$nick is checking the status of $arg1 \($arg2 \($chan\)\)."
		split_putchan $chan "Checking the status of $arg1 \($arg2\)."
		if {[catch {set sock [socket -async $arg1 $arg2]}]} { split_status_output split_putchan $chan $arg1 $arg2 "was refused." ; return }
		fileevent $sock writable [list split_connected 0 0 $chan $sock $arg1 $arg2 [utimer 15 [list split_timeout 0 0 $chan $sock $arg1 $arg2]]]
	} elseif {[split_ncm links $arg0] && [matchattr $hand $split_setting(flag_links)]} {
		split_logcast "$nick requested a links update. \($chan\)"
		putserv "LINKS"
		split_putchan $chan "The links list has been updated."
		return
	} {
		split_putnotc $nick "Usage: $split_setting(command_pub) <command>"
		split_putnotc $nick "See $split_setting(command_pub) help for a list of public commands."
	}
}
proc split_nflaglist {nick uhost hand chan text} {
	global split_init_server split_connect_server split_setting split_masterservers split_linkedservers split_splittime split_connectionstatus split_services_split
	split_logcast "$nick requested netsplit list. \($chan\)"
	if {![info exists split_masterservers] || ![info exists split_linkedservers] || $split_init_server || $split_connect_server} { split_putchan $chan "The server databases do not exist. Please wait until the next automatic update occurs." ; return }
	if {[array size split_masterservers] == [array size split_linkedservers] && (!$split_setting(services_check) || (![info exists split_services_split(stats)] && ![info exists split_services_split(stats)]) || ![split_checks all] || [expr ($split_services_split(services) + $split_services_split(stats)) == 0])} { split_putchan $chan "There are no split servers." ; return }
	set expr 0
	if {$split_setting(services_check)} {
		if {[info exists split_services_split(services)] && [split_checks services]} { set expr $split_services_split(services) }
		if {[info exists split_services_split(stats)] && [split_checks stats]} { set expr [expr $expr + $split_services_split(stats)] }
	}
	[split_queue [set queue [expr (([array size split_masterservers] - [array size split_linkedservers] + $expr) > 3)?0:1]]] "PRIVMSG $chan :Current netsplits:"
	foreach s [array names split_masterservers] {
		if {![info exists split_linkedservers($s)]} { [split_queue $queue] "PRIVMSG $chan :$s \(from $split_masterservers($s)\) \(Connection: $split_connectionstatus($s)\) \(Split: [clock format $split_splittime($s) -format "%I:%M:%S %p \(%D\)"]\)" }
	}
	if {$split_setting(services_check)} {
		if {[info exists split_services_split(services)] && [split_checks services] && $split_services_split(services)} { [split_queue $queue] "PRIVMSG $chan :$split_setting(services_servicesname) \(from \(Unknown\)\) \(Split: [clock format $split_splittime($split_setting(services_servicesname)) -format "%I:%M:%S %p \(%D\)"]\)" }
		if {[info exists split_services_split(stats)] && [split_checks stats] && $split_services_split(stats)} { [split_queue $queue] "PRIVMSG $chan :$split_setting(services_statsname) \(from \(Unknown\)\) \(Split: [clock format $split_splittime($split_setting(services_statsname)) -format "%I:%M:%S %p \(%D\)"]\)" }
	}
	[split_queue $queue] "PRIVMSG $chan :End of list."
}
proc split_raw {from keyword text} {
	global split_setting split_lusers_data split_uplink split_linkedservers split_splittime split_localserver split_services_split split_hops split_splitstats split_connectionstatus split_infoline split_init_server split_masterservers split_incoming
	switch -regexp -- $keyword {
		"251|255|265|266" { set split_lusers_data($keyword) [split_lusersleft 0 $text] }
		"252|254" { set split_lusers_data($keyword) [split_lusersleft 1 $text] }
		"351" {
			if {!$split_setting(services_check)} { return }
			if {[split_checks services] && [split_cm $split_setting(services_servicesname) $from]} {
				foreach {a b} [list uplink(Unknown) $split_localserver uplink($split_setting(services_servicesname)) Unknown] { set split_${a} $b }
				if {![info exists split_services_split(services)]} {
					regsub -all -- "%server" $split_setting(newlink_prefix) $split_setting(services_servicesname) msg
					regsub -all -- "%linkedto" $msg "\(Unknown\)" msg
					set split_services_split(services) 0
					split_notify $msg
					split_putsvrlog 4 $split_setting(services_servicesname) "\(Unknown\)"
				} elseif {$split_services_split(services)} {
					regsub -all -- "%server" $split_setting(rejoin_prefix) $split_setting(services_servicesname) msg
					regsub -all -- "%splitfrom" $msg "\(Unknown\)" msg
					regsub -all -- "%time" $msg [clock format $split_splittime($split_setting(services_servicesname)) -format "%I:%M:%S %p \(%D\)"] msg
					regsub -all -- "%duration" $msg [duration [expr [unixtime] - $split_splittime($split_setting(services_servicesname))]] msg
					set split_services_split(services) 0
					split_notify $msg
					split_putsvrlog 3 $split_setting(services_servicesname) "\(Unknown\)"
				}
			} elseif {[split_checks stats] && [split_cm $split_setting(services_statsname) $from]} {
				foreach {a b} [list uplink(Unknown) $split_localserver uplink($split_setting(services_statsname)) Unknown] { set split_${a} $b }
				if {![info exists split_services_split(stats)]} {
					regsub -all -- "%server" $split_setting(newlink_prefix) $split_setting(services_statsname) msg
					regsub -all -- "%linkedto" $msg "\(Unknown\)" msg
					set split_services_split(stats) 0
					split_notify $msg
					split_putsvrlog 4 $split_setting(services_statsname) "\(Unknown\)"
				} elseif {$split_services_split(stats)} {
					regsub -all -- "%server" $split_setting(rejoin_prefix) $split_setting(services_statsname) msg
					regsub -all -- "%splitfrom" $msg "\(Unknown\)" msg
					regsub -all -- "%time" $msg [clock format $split_splittime($split_setting(services_statsname)) -format "%I:%M:%S %p \(%D\)"] msg
					regsub -all -- "%duration" $msg [duration [expr [unixtime] - $split_splittime($split_setting(services_statsname))]] msg
					set split_services_split(stats) 0
					split_notify $msg
					split_putsvrlog 3 $split_setting(services_statsname) "\(Unknown\)"
				}
			}
		}
		"364" {
			set split_localserver $from
			set text [split $text]
			foreach {a b} [list incoming [lindex $text 2] uplink [lindex $text 2] hops [split [string trimleft [join [lindex $text 3]] :]] infoline [lrange $text 4 end]] { set split_${a}([lindex [split $text] 1]) $b }
		}
		"365" {
			foreach {a b} [list split_localserver $from split_init_server 0 nonew 0] { set $a $b }
			if {![array exists split_linkedservers]} {
				foreach s [array names split_incoming] { set split_linkedservers($s) $split_incoming($s) }
				set nonew 1
			}
			foreach s [array names split_linkedservers] {
				if {[info exists split_incoming($s)]} { continue }
				if {![info exists split_splitstats($s)]} { set split_splitstats($s) 0 }
				incr split_splitstats($s)
				set split_splittime($s) [clock seconds]
				regsub -all -- "%server" $split_setting(connrefused_msg) $s msg
				regsub -all -- "%splitfrom" $msg $split_masterservers($s) msg
				if {[catch {set sock [socket -async $s $split_setting(default_port)]}]} {
					set split_connectionstatus($s) "Refused"
					split_notify $msg
					split_putsvrlog 1 $s $split_masterservers($s)
				} { fileevent $sock writable [list split_connected 0 1 -1 $sock $s $split_setting(default_port) [utimer 15 [list split_timeout 0 1 -1 $sock $s $split_setting(default_port)]]] }
				foreach i [list linkedservers uplink hops infoline] { catch { unset split_${i}($s) } }
			}
			foreach s [array names split_incoming] {
				if {![info exists split_masterservers($s)] && ![info exists split_linkedservers($s)] && !$nonew} {
					regsub -all -- "%server" $split_setting(newlink_prefix) $s msg
					regsub -all -- "%linkedto" $msg $split_incoming($s) msg
					split_notify $msg
					split_putsvrlog 4 $s $split_incoming($s)
				} elseif {[info exists split_masterservers($s)] && ![info exists split_linkedservers($s)]} {
					regsub -all -- "%server" $split_setting(rejoin_prefix) $s msg
					regsub -all -- "%splitfrom" $msg $split_masterservers($s) msg
					regsub -all -- "%time" $msg [clock format $split_splittime($s) -format "%I:%M:%S %p \(%D\)"] msg
					regsub -all -- "%duration" $msg [duration [expr [unixtime] - $split_splittime($s)]] msg
					split_notify $msg
					split_putsvrlog 3 $s $split_incoming($s)
				} elseif {![split_cm $split_linkedservers($s) $split_incoming($s)] && ![split_cm $s $split_linkedservers($s)] && ![split_cm $s $split_incoming($s)]} {
					regsub -all -- "%server" $split_setting(relink_prefix) $s msg
					regsub -all -- "%oldlink" $msg $split_linkedservers($s) msg
					regsub -all -- "%newlink" $msg $split_incoming($s) msg
					split_notify $msg
					split_putsvrlog 2 $s $split_linkedservers($s) $split_incoming($s)
				}
				set split_masterservers($s) $split_incoming($s)
				set split_linkedservers($s) $split_incoming($s)
			}
			if {[info exists split_incoming]} { unset split_incoming }
			if {$split_setting(services_check)} {
				if {[split_checks services]} { putserv "VERSION $split_setting(services_servicesname)" }
				if {[split_checks stats]} { putserv "VERSION $split_setting(services_statsname)" }
			}
			putserv "LUSERS"
		}
		"402" {
			if {!$split_setting(services_check)} { return }
			if {[split_checks services] && [split_cm $split_setting(services_servicesname) $from] && [info exists split_services_split(services)] && !$split_services_split(services)} {
				regsub -all -- "%server" $split_setting(connfail_msg) $split_setting(services_servicesname) msg
				regsub -all -- "%splitfrom" $msg "\(Unknown\)" msg
				if {![info exists split_splitstats($split_setting(services_servicesname))]} { set split_splitstats($split_setting(services_servicesname)) 0 }
				foreach {a b} [list split_splittime($split_setting(services_servicesname)) [clock seconds] split_services_split(services) 1 split_connectionstatus($split_setting(services_servicesname)) -] { set $a $b }
				incr split_splitstats($split_setting(services_servicesname))
				if {[info exists split_uplink($split_setting(services_servicesname))]} { unset split_uplink($split_setting(services_servicesname)) }
				split_notify $msg
				split_putsvrlog 1 $split_setting(services_servicesname) "\(Unknown\)"
			} elseif {[split_checks services] && [split_cm $split_setting(services_statsname) $from] && [info exists split_services_split(stats)] && !$split_services_split(stats)} {
				regsub -all -- "%server" $split_setting(connfail_msg) $split_setting(services_statsname) msg
				regsub -all -- "%splitfrom" $msg "\(Unknown\)" msg
				if {![info exists split_splitstats($split_setting(services_statsname))]} { set split_splitstats($split_setting(services_statsname)) 0 }
				foreach {a b} [list split_splittime($split_setting(services_statsname)) [clock seconds] split_services_split(stats) 1 split_connectionstatus($split_setting(services_statsname)) -] { set $a $b }
				incr split_splitstats($split_setting(services_statsname))
				if {[info exists split_uplink($split_setting(services_statsname))]} { unset split_uplink($split_setting(services_statsname)) }
				split_notify $msg
				split_putsvrlog 1 $split_setting(services_statsname) "\(Unknown\)"
			}
		}
	}
}
proc split_status_output {type chan server port conmsg} {
	global split_linkedservers split_setting split_masterservers split_uplink split_splittime split_splitstats split_connectionstatus split_services_split
	split_debug "split_status_output(chk1)"
	if {$split_setting(services_check)} {
		if {[info exists split_services_split(services)] && [split_checks services] && [split_cm $server $split_setting(services_servicesname)]} { $type $chan [expr $split_services_split(services)?"$server is currently split from \(Unknown\). \([clock format $split_splittime($server) -format {%I:%M:%S %p \(%D\)}]\). It has been split a total of $split_splitstats($server) times. The current connection to $server \($port\) $conmsg":"$server is currently linked to \(Unknown\). The current connection to $server \($port\) $conmsg"] ; return }
		if {[info exists split_services_split(stats)] && [split_checks stats] && [split_cm $server $split_setting(services_statsname)]} { $type $chan [expr $split_services_split(stats)?"$server is currently split from \(Unknown\). \([clock format $split_splittime($server) -format {%I:%M:%S %p \(%D\)}]\). It has been split a total of $split_splitstats($server) times. The current connection to $server \($port\) $conmsg":"$server is currently linked to \(Unknown\). The current connection to $server \($port\) $conmsg"] ; return }
	}
	if {[info exists split_linkedservers($server)]} { $type $chan "$server is currently linked to $split_uplink($server). The current connection to $server \($port\) $conmsg" ; return }
	if {[info exists split_masterservers($server)]} { $type $chan "$server is currently split from $split_masterservers($server) \([clock format $split_splittime($server) -format "%I:%M:%S %p \(%D\)"]\). It has been split a total of $split_splitstats($server) times. At the time of the split, the connection status was \"$split_connectionstatus($server).\" The current connection to $server \($port\) $conmsg" ; return }
	$type $chan "$server is not a known server. The current connection to $server \($port\) $conmsg"
}
proc split_timeout {type auto idx sock host port} {
	global split_connectionstatus split_setting split_masterservers
	split_debug "split_timeout(chk1)"
	close $sock
	if {!$auto} { split_status_output [expr $type?"split_putdcc":"split_putchan"] $idx $host $port "timed out." ; return }
	regsub -all -- "%server" $split_setting(conntimeout_msg) $host msg
	regsub -all -- "%splitfrom" $msg $split_masterservers($host) msg
	set split_connectionstatus($host) "Timeout"
	split_notify $msg
	split_putsvrlog 1 $host $split_masterservers($host)
}
proc split_connected {type auto idx sock host port timerid} {
	global split_setting split_connectionstatus split_masterservers
	split_debug "split_connected(chk1)"
	killutimer $timerid
	fileevent $sock writable {}
	fileevent $sock readable {}
	set failed [expr {([set error [fconfigure $sock -error]] == "")?0:1}]
	close $sock
	if {!$auto} { split_status_output [expr $type?"split_putdcc":"split_putchan"] $idx $host $port [expr $failed?"failed.":"was accepted."] ; return }
	regsub -all -- "%server"  $host msg
	regsub -all -- "%splitfrom" $msg $split_masterservers($host) msg
	set split_connectionstatus($host) [expr {($failed)?"Failed":"Accepted"}]
	split_notify $msg
	split_putsvrlog 1 $host $split_masterservers($host)

}
proc split_run_maintenance {min hour day month year} {
	global split_setting split_database split_masterservers split_linkedservers
	if {$split_setting(auto_backup)} {
		split_putlog "Backing up databases."
		foreach d [array names split_database] {
			if {[catch [file copy -force $split_database($d) $split_database($d).bak]]} { split_putlog "Could not backup $split_database($d)." }
		}
	}
	if {$split_setting(auto_clearlog)} {
		split_putlog "Clearing logs."
		if {[catch [file delete -force $split_database(log)]]} { split_putlog "The log file could not be deleted." }
	}
	split_putsvrlog 5 "Backing&up&databases&and&clearing&logs."
}
proc split_update {} {
	global split_setting split_connect_server
	if {!$split_connect_server} { putserv "LINKS" }
	if {[lsearch -glob [utimers] "* split_update *"] == -1} { utimer $split_setting(t_update) split_update }
}
proc split_save_databases {} {
	global split_masterservers split_services_split split_linkedservers split_splitstats split_database split_splittime split_connectionstatus split_setting
	foreach a {linkedservers connectionstatus splitstats splittime masterservers lusers_data} { split_savearray split_$a $split_database($a) }
	if {$split_setting(services_check)} { split_savearray split_services_split $split_database(services) }
}
proc split_putsvrlog {type {server ""} {from ""} {text ""}} {
	global split_database split_connectionstatus
	set time [clock format [clock seconds] -format "%D %I:%M %p"]
	set log [open $split_database(log) a+]
	if {$server != ""  && [info exists split_connectionstatus($server)]} { regsub -all -- " " $split_connectionstatus($server) "\\&" arg1 }
	switch -- $type {
		1 { puts $log "$time Netsplit ${server}&from&${from}. $arg1" }
		2 { puts $log "$time Relink ${server}&from&${from}&to&${text}. -" }
		3 { puts $log "$time Rejoin ${server}&to&${from}. -" }
		4 { puts $log "$time New&Link ${server}&to&${from}. -" }
		5 { puts $log "$time Maintenance ${server}&${from}&${text} -" }
	}
	close $log
	split_create_html
	split_save_databases
}
proc split_services_cleanup {type} {
	global split_services_split split_setting split_splitstats split_linkedservers split_splittime split_infoline split_uplink split_hops split_masterservers split_connectionstatus
	if {$type == 1} {
		foreach i [list $split_setting(services_servicesname) $split_setting(services_statsname) Unknown] {
			catch { unset split_services_split($i) split_splitstats($i) split_uplink($i) split_linkedservers($i) split_masterservers($i) split_splittime($i) split_infoline($i) split_hops($i) split_connectionstatus($i) }
		}
		catch { unset split_services_split }
		return
	}
	foreach i [list $split_setting(services_servicesname) $split_setting(services_statsname)] {
		foreach var {splitstats uplink linkedservers masterservers splittime infoline hops connectionstatus} {
			if {$i == "" && [info exists split_${var}($i)]} { unset split_${var}($i) }
		}
	}
	if {![split_checks stats] && [info exists split_services_split(stats)]} { unset split_services_split(stats) }
	if {![split_checks services] && [info exists split_services_split(services)]} { unset split_services_split(services) }
}
proc split_create_html {} {
	global split_lusers_data split_init_server split_connect_server split_services_split split_setting server split_splitstats split_linkedservers split_splittime split_infoline split_uplink split_hops split_masterservers split_connectionstatus split_localserver split_database
	if {$split_setting(html_disable) || [expr ($split_setting(Page_Network_Users) + $split_setting(Page_Linked_Servers) + $split_setting(Page_Split_Servers) + $split_setting(Page_Split_Graph) + $split_setting(Page_Network_Map) + $split_setting(Page_Activity_Log)) <= 0] || $split_init_server || $split_connect_server} { return }
	foreach {a b} [list splitnum 0 curlinked [array size split_linkedservers] totalservers [array size split_masterservers] time [clock format [clock seconds] -format {%I:%M %p}] date [clock format [clock seconds] -format %D]] { set $a $b }
	split_services_cleanup [expr $split_setting(services_check)?0:1]
	if {$split_setting(services_check)} {
		incr totalservers [expr [split_checks services] + [split_checks stats]]
		if {[info exists split_services_split(services)]} { incr [expr $split_services_split(services)?"splitnum":"curlinked"] }
		if {[info exists split_services_split(stats)]} { incr [expr $split_services_split(stats)?"splitnum":"curlinked"] }
		if {[split_checks services] && ![info exists split_splitstats($split_setting(services_servicesname))]} { set split_splitstats($split_setting(services_servicesname)) [expr [info exists split_uplink($split_setting(services_servicesname))]?0:1] }
		if {[split_checks stats] && ![info exists split_splitstats($split_setting(services_statsname))]} { set split_splitstats($split_setting(services_statsname)) [expr [info exists split_uplink($split_setting(services_statsname))]?0:1] }
	}
	foreach s [array names split_masterservers] {
		if {![info exists split_linkedservers($s)]} { incr splitnum }
		if {![info exists split_splitstats($s)]} { set split_splitstats($s) [expr [info exists split_linkedservers($s)]?0:1] }
	}
	set netstats [open $split_setting(netstats_page) w]
	puts $netstats "<html>\n  <head>\n    <title>$split_setting(Page_TitleTag)</title>\n    <META NAME=\"description\" CONTENT=\"Network Statistics\">\n    <META NAME=\"keywords\" CONTENT=\"netsplit, netstats, stats, irc, IRCd, link\">\n  </head>\n\n  <body bgcolor=\"$split_setting(Page_BGC)\" link=\"$split_setting(Page_LinkC)\" alink=\"$split_setting(Page_LinkC)\" vlink=\"$split_setting(Page_LinkC)\">"
	if {$split_setting(Page_Img) != ""} { puts $netstats "    <center><img src=\"$split_setting(Page_Img)\"></img></center>" }
	puts $netstats "    <center><font size=\"$split_setting(Page_TitleS)\" color=\"$split_setting(Page_TTextC)\">$split_setting(Page_Title)</font></center>\n    <br>\n    <center><font size=\"$split_setting(Page_TCS)\" color=\"$split_setting(Page_TextC)\">Connected to $split_localserver on port [lindex [split $server :] 1].</font></center>\n    <br>"
	if {$split_setting(Page_Linked_Servers)} {
		set servernum 0
		puts $netstats "    <br>\n    <br>\n    [split_format_td title 0 1 {Linked Servers}]\n    <br>\n    <table align=\"center\">\n      <tr>"
		foreach {a b} [list 1 Number 2 Server 1 Uplink 2 Hops 1 {Info Line} 2 {Total Splits}] { puts $netstats "        [split_format_td header $a 1 $b]" }
		puts $netstats "      </tr>"
		foreach s [array names split_linkedservers] {
			incr servernum
			foreach a [list up hops infoline sstat] { set $a "-" }
			catch {
				set up [split_mIRC_link Linked_Servers $split_uplink($s)]
				set hops $split_hops($s)
				set infoline [join $split_infoline($s)]
				set sstat $split_splitstats($s)
			}
			puts $netstats "      <tr>"
			foreach {a b} [list 1 $servernum 2 [split_mIRC_link Linked_Servers $s] 1 $up 2 $hops 1 $infoline 2 $sstat] { puts $netstats "        [split_format_td cell $a 1 $b]" }
			puts $netstats "      </tr>"
		}
		if {$split_setting(services_check)} {
			if {[split_checks services] && [info exists split_services_split(services)] && !$split_services_split(services)} {
				set sstat "-"
				if {[info exists split_splitstats($split_setting(services_servicesname))]} { set sstat $split_splitstats($split_setting(services_servicesname)) }
				incr servernum
				puts $netstats "      <tr>"
				foreach {a b} [list 1 $servernum 2 [split_mIRC_link Linked_Servers $split_setting(services_servicesname)] 1 - 2 - 1 - 2 $sstat] { puts $netstats "        [split_format_td cell $a 1 $b]" }
				puts $netstats "      </tr>"
			}
			if {[split_checks stats] && [info exists split_services_split(stats)] && !$split_services_split(stats)} {
				set sstat "-"
				if {[info exists split_splitstats($split_setting(services_statsname))]} { set sstat $split_splitstats($split_setting(services_statsname)) }
				incr servernum
				puts $netstats "      <tr>"
				foreach {a b} [list 1 $servernum 2 [split_mIRC_link Linked_Servers $split_setting(services_statsname)] 1 - 2 - 1 - 2 $sstat] { puts $netstats "        [split_format_td cell $a 1 $b]" }
				puts $netstats "      </tr>"
			}
		}
		puts $netstats "    </table>\n    <br>\n    <br>"
	}
	if {$split_setting(Page_Split_Servers)} {
		set servernum 0
		puts $netstats "    [split_format_td title 0 1 {Split Servers}]\n    <br>\n    <table align=\"center\">\n      <tr>"
		foreach {a b} [list 1 Number 2 Server 1 From 2 {Split Time} 1 Connection 2 {Total Splits}] { puts $netstats "        [split_format_td header $a 1 $b]" }
		puts $netstats "      </tr>"
		if {[array size split_masterservers] == [array size split_linkedservers] && (!$split_setting(services_check) || (![info exists split_services_split(stats)] && ![info exists split_services_split(stats)]) || ![split_checks all] || [expr ($split_services_split(services) + $split_services_split(stats)) == 0])} {
			puts $netstats "      <tr>"
			foreach a [list 1 2 1 2 1 2] { puts $netstats "        [split_format_td cell $a 1 -]" }
			puts $netstats "      </tr>"
		} {
			foreach s [array names split_masterservers] {
				if {[info exists split_linkedservers($s)]} { continue }
				incr servernum
				foreach {a b} [list master - stime - connection - sstat 0] { set $a $b }
				catch {
					set master [split_mIRC_link Split_Servers $split_masterservers($s)]
					set stime [clock format $split_splittime($s) -format "%I:%M:%S %p \(%D\)"]
					set connection $split_connectionstatus($s)
					set sstat $split_splitstats($s)
				}
				puts $netstats "      <tr>"
				foreach {a b} [list 1 $servernum 2 [split_mIRC_link Split_Servers $s] 1 $master 2 $stime 1 $connection 2 $sstat] { puts $netstats "        [split_format_td cell $a 1 $b]" }
				puts $netstats "      </tr>"
			}
			if {$split_setting(services_check)} {
				if {[info exists split_services_split(services)] && $split_services_split(services)} {
					incr servernum
					set sstime "-"
					set sstat "0"
					catch {
						set sstime [clock format $split_splittime($split_setting(services_servicesname)) -format "%I:%M:%S %p \(%D\)"]
						set sstat $split_splitstats($split_setting(services_servicesname))
					}
					puts $netstats "      <tr>"
					foreach {a b} [list 1 $servernum 2 [split_mIRC_link Split_Servers $split_setting(services_servicesname)] 1 - 2 $stime 1 - 2 $sstat] { puts $netstats "        [split_format_td cell $a 1 $b]" }
					puts $netstats "      </tr>"
				}
				if {[info exists split_services_split(stats)] && $split_services_split(stats)} {
					incr servernum
					set sstime "-"
					set sstat "0"
					catch {
						set sstime [clock format $split_splittime($split_setting(services_statsname)) -format "%I:%M:%S %p \(%D\)"]
						set sstat $split_splitstats($split_setting(services_statsname))
					}
					puts $netstats "      <tr>"
					foreach {a b} [list 1 $servernum 2 [split_mIRC_link Split_Servers $split_setting(services_statsname)] 1 - 2 $stime 1 - 2 $sstat] { puts $netstats "        [split_format_td cell $a 1 $b]" }
					puts $netstats "      </tr>"
				}
			}
		}
		puts $netstats "    </table>\n    <br>\n    <br>"
	}
	if {$split_setting(Page_Split_Graph)} {
		foreach {a b} [list counti 0 maxsplits "" maxsplitscount 0 totalsplits 0] { set $a $b }
		foreach i [array names split_splitstats] {
			if {![info exists split_masterservers($i)]} { continue }
			if {$split_splitstats($i) >= $maxsplitscount} { set maxsplitscount $split_splitstats($i) }
			lappend maxsplits [list $i $split_splitstats($i)]
		}
		set maxsplits [lsort -decreasing -integer -index 1 $maxsplits]
		foreach w [join $maxsplits] { if {[split_notnumber $w]} { lappend maxsplits2 $w } }
		set maxsplits $maxsplits2
		puts $netstats "    [split_format_td title 0 1 {Split Graph}]\n    <br>\n    <table align=\"center\">\n      <tr>"
		foreach {a b} [list 1 Number 2 Server 1 Graph 2 Splits] { puts $netstats "        [split_format_td header $a 1 $b]" }
		puts $netstats "      </tr>"
		if {$maxsplitscount == 0} {
			puts $netstats "      <tr>"
			foreach a {1 2 1 2} { puts $netstats "        [split_format_td cell $a 1 -]" }
			puts $netstats "      </tr>"
		} {
			foreach i $maxsplits {
				if {[llength $maxsplits] <= $counti} { break }
				incr counti
				if {![info exists split_splitstats($i)]} { set split_splitstats($i) 0 }
				puts $netstats "      <tr>\n        [split_format_td cell 1 1 $counti]\n        [split_format_td cell 2 1 [lindex $maxsplits [expr $counti - 1]]]\n        [split_format_td cell 1 0 [join [list <img src=\"$split_setting(Page_GImg)\" height=\"10\" width=\"[expr round($split_splitstats($i)/double($maxsplitscount)*360)]\"></img>]]]\n        [split_format_td cell 2 1 $split_splitstats([lindex $maxsplits [expr $counti - 1]])]\n      </tr>"
			}
		}
		puts $netstats "    </table>\n    <br>\n    <br>"
	}
	if {$split_setting(Page_Network_Users)} {
		puts $netstats "    [split_format_td title 0 1 {Network Users}]\n    <br>\n    <table align=\"center\">\n      <tr>[split_format_td header 1 1 {Network Users}]</tr>\n      <tr>\n        <td align=\"left\" bgcolor=\"$split_setting(Page_TCC_1)\"><font size=\"$split_setting(Page_TCS)\" color=\"$split_setting(Page_CC)\">\n<pre>\n"
		if {![info exists split_lusers_data] || [array size split_lusers_data] == 0} {
			puts $netstats "No data exists."
		} { foreach i [array names split_lusers_data] { puts $netstats "   $split_lusers_data($i).   " } }
		puts $netstats "</pre>\n      </tr>\n    </table>"
	}
	if {$split_setting(Page_Network_Map)} {
		foreach v [list avghops totalsplits] { set $v 0 }
		foreach a [array names split_splitstats] { incr totalsplits $split_splitstats($a) }
		foreach s [array names split_hops] { incr avghops $split_hops($s) }
		set avghops [expr $avghops / [array size split_hops]]
		puts $netstats "    [split_format_td title 0 1 {Network Map}]\n    <br>\n    <table align=\"center\">\n      <tr>\n        [split_format_td header 1 1 Map]\n      </tr>\n      <tr>\n        <td align=\"left\" bgcolor=\"$split_setting(Page_TCC_1)\"><font size=\"$split_setting(Page_TCS)\" color=\"$split_setting(Page_CC)\">\n<pre>\n\n   [split_mIRC_link Network_Map $split_localserver]     "
		split_netmap puts "     " $netstats $split_localserver
		puts $netstats "</pre>\n          <center>Total Linked: $curlinked - Total Splits: $totalsplits - Average Hops: $avghops</center>\n          <br>\n        </td>\n      </tr>\n    </table>\n    <br>\n    <br>"
	}
	if {$split_setting(Page_Activity_Log)} {
		puts $netstats "    [split_format_td title 0 1 {Activity Log}]\n    <br>\n    <table align=\"center\">\n      <tr>"
		foreach {a b} [list 1 Date 2 Time 1 Event 2 Data 1 Connection] { puts $netstats "        [split_format_td header $a 1 $b]" }
		puts $netstats "      </tr>"
		if {[catch {open $split_database(log) r} log]} {
			puts $netstats "      <tr>"
			foreach {a b c} [list 1 1 $date 2 1 $time 1 1 Maintenance 2 0 [expr [file exists $split_database(log)]?"An error occurred while accessing the logfile.":"A new logfile has been started."] 1 1 -] { puts $netstats "        [split_format_td cell $a $b $c]" }
			puts $netstats "      </tr>"
		} {
			foreach line [split [read $log] \n] {
				if {[string length [join $line]] <= 0} { continue }
				for {set i 0} {$i <= 5} {incr i} { regsub -all -- "\\&" [join [lindex $line $i]] " " arg[expr $i + 1] }
				puts $netstats "      <tr>"
				foreach {a b c} [list 1 1 $arg1 2 1 [list $arg2 $arg3] 1 1 $arg4 2 0 $arg5 1 1 $arg6] { puts $netstats "        [split_format_td cell $a $b $c]" }
				puts $netstats "      </tr>"
			}
			close $log
		}
		puts $netstats "    </table>\n    <br>\n    <br>"
	}
	puts $netstats "    <center><font color=\"$split_setting(Page_TextC)\">Last Updated: $date $time \(Data Recording Started: [clock format $split_splittime(start) -format "%D %I:%M %p"]\)</font></center>\n  </body>\n</html>\n<!--<DAWGID>Split $split_setting(version)</DAWGID>-->"
	close $netstats
}
proc split_netmap {action indent fd server} {
	global split_uplink
	foreach server2 [array names split_uplink] {
		if {![split_cm $server $server2] && [split_cm $split_uplink($server2) $server]} { lappend downlinks $server2 }
	}
	if {![info exists downlinks]} { return }
	foreach server2 $downlinks {
		if {[split_cm $server2 [lindex $downlinks end]]} {
			$action $fd "$indent `-[expr [split_cm $action puts]?{[split_mIRC_link Network_Map $server2]}:{$server2}]     "
			split_netmap $action "$indent  " $fd $server2
		} {
			$action $fd "$indent |-[expr [split_cm $action puts]?{[split_mIRC_link Network_Map $server2]}:{$server2}]     "
			split_netmap $action "$indent |   " $fd $server2
		}
	}
}
putlog "\002SPLIT:\002 Split.tcl Version $split_setting(version) by Wcc is loaded."
